﻿////////////////////////////////////////////////////////////////////////////////
// TestDEXi: A console application demonstrating typical use cases of the
//           DEXi.NET library (http://kt.ijs.si/MarkoBohanec/dexinet.html)
//
//           Author: Marko Bohanec
//           Version: 1.0
//
//
//TestDEXi is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public
//License as published by the Free Software Foundation; either
//version 2.1 of the License, or (at your option) any later version.
//
//This software is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
//Lesser General Public License for more details.
//
//You should have received a copy of the GNU Lesser General Public
//License along with this library; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
////////////////////////////////////////////////////////////////////////////////
//Version 1.0 2017-03-20:
//      Ported to C# from java, JDEXi version 3.1
////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using DEXi;

namespace TestDEXi
{

    /// <summary>
    /// Test DEXi.NET library and show how to program typical operations on DEXi models 
    /// </summary>
    class TestDEXi
    {
        static int Main(string[] args) {
            try {
                if (args.Length != 1) {
                    Console.WriteLine("Usage: TestDEXi input_DEXi_model.dxi");
                    return 1;
                }

                // load the model from a .dxi file
                Model model = new Model(args[0]);

                // print lists: basic, aggregate and linked attributes
                Console.WriteLine("\nAttributes:");
                Console.WriteLine("  Basic: " + model.AttributesToString(model.Basic));
                Console.WriteLine("  Aggregate: " + model.AttributesToString(model.Aggregate));
                Console.WriteLine("  Linked: " + model.AttributesToString(model.Linked));
                Console.WriteLine("  AllAttributes: " + model.AttributesToString(model.AllAttributes));

                List<DEXi.Attribute> all = model.AllAttributes;

                // calculate maxnl = maximum number of character occupied by attribute name + level indent
                int maxnl = 0;
                for (int i = 0; i < all.Count; i++) {
                    int nl = all[i].Name.Length + 2 * all[i].Level;
                    if (nl > maxnl) { maxnl = nl; }
                }
                maxnl = maxnl + 1; // + gap

                // print attribute structure with descriptions
                Console.WriteLine("\nAttributes and descriptions:");

                for (int i = 0; i < all.Count; i++) {
                    DEXi.Attribute att = all[i];
                    string name = att.Name;
                    int len = name.Length;
                    int lev = 2 * att.Level;
                    Console.WriteLine(EmptyStr(lev) + name + EmptyStr(maxnl - len - lev) + att.Description);
                }

                // print attribute structure with scales
                Console.WriteLine("\nAttributes and scales:");

                for (int i = 0; i < all.Count; i++) {
                    DEXi.Attribute att = all[i];
                    string name = att.Name;
                    int len = name.Length;
                    int lev = 2 * att.Level;
                    string scl = "";
                    for (int s = 0; s < att.ScaleSize; s++) {
                        string val = att.ScaleValue(s);
                        if (scl != "") { scl = scl + ";"; }
                        scl = scl + val;
                    }
                    Console.WriteLine(EmptyStr(lev) + name + EmptyStr(maxnl - len - lev) + scl);
                }

                // print root attribute and its descendants
                DEXi.Attribute root = all[0];

                if (root != null) {
                    Console.WriteLine("\nRoot attribute " + root.Name + " has " + root.AttributeCount + " attribute(s):");
                    for (int i = 0; i < root.AttributeCount; i++) {
                        DEXi.Attribute a = root.Attributes[i];
                        Console.WriteLine("  Attribute " + i + "=" + a.Name);
                    }

                    // print root function, if exists
                    if (root.RuleCount >= 0) {
                        DumpFunction(root);
                        DumpBounds(root);
                    }

                    // create a random alternative
                    Console.WriteLine("\nEvaluating a random alternative");
                    Random rnd = new Random();
                    model.ClearInputValues();
                    for (int i = 0; i < model.Basic.Count; i++) {
                        model.SetInputValue(i, rnd.Next(model.Basic[i].ScaleSize));
                    }

                    // evaluate
                    model.Evaluate(Model.Evaluation.SET, true);

                    // print evaluation results
                    // ... quickly
                    Console.WriteLine("\nInput values:  " + model.InputValues);
                    Console.WriteLine("Output values: " + model.OutputValues);

                    // ... with attribute structure
                    Console.WriteLine("\nAggregate attributes and values:");
                    for (int i = 0; i < model.Aggregate.Count; i++) {
                        DEXi.Attribute att = model.Aggregate[i];
                        string name = att.Name;
                        int len = name.Length;
                        int lev = 2 * att.Level;
                        Console.WriteLine(EmptyStr(lev) + name + EmptyStr(maxnl - len - lev)  + model.OutputValuesString(i));
                    }

                }
            } catch (Exception e) {
                Console.WriteLine("TestDEXi: Exception: " + e.Message);
            }
            return 0;
        }

        public static void DumpFunction(DEXi.Attribute a) {
            Console.Write("\nFunction of " + a.Name + " depends on:");
            for (int j = 0; j < a.AttributeCount; j++) {
                DEXi.Attribute x = a.Attributes[j];
                Console.Write(" " + x.Name);
            }
            Console.WriteLine();
            for (int r = 0; r < a.RuleCount; r++) {
                Rule rule = a.Rule(r);
                int[] args = a.IndexToArgs(r);
                Rule ruleByArgs = a.FunctionValue(args);
                string msg = (rule == ruleByArgs) ? " " : " !error! ";
                Console.Write(r + ": " + args.ToString() + " ");
                Console.Write(rule.Low + ":" + rule.High + msg);
                for (int j = 0; j < a.AttributeCount; j++) {
                    DEXi.Attribute x = a.Attributes[j];
                    Console.Write(" " + x.ScaleValue(args[j]));
                }
                Console.WriteLine(" -> " + a.ScaleValue(rule.Low) + ":" + a.ScaleValue(rule.High));
            }
        }

        public static void DumpBounds(DEXi.Attribute a) {
            Console.WriteLine("\nAdmissible bounds of function " + a.Name);
            for (int r = 0; r < a.RuleCount; r++) {
                Rule rule = a.Rule(r);
                Rule bounds = a.RuleBounds(r);
                int[] args = a.IndexToArgs(r);
                Console.Write(r + ": " + args.ToString() + " ");
                Console.Write(rule.Low + ":" + rule.High);
                Console.Write("  [" + bounds.Low + ":" + bounds.High + "]");
                if (!a.RuleIsConsistent(r, bounds)) {
                    Console.Write(" inconsistent");
                }
                Console.WriteLine();
            }
            if (!a.FunctionIsConsistent()) {
                Console.WriteLine("Warning: function is inconsistent");
            }
        }

        public static string EmptyStr(int n) {
            if (n <= 0) { return ""; }
            else { return new string(' ',n); }
        }
    }
}
